<?php

namespace Zeedhi\ReportWebService\Controllers;

use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\ReportWebService\SpreadSheet\XLSXWriter;
use Zeedhi\ReportWebService\Services\CSVService;
use Zeedhi\ReportWebService\Services\Report;
use Zeedhi\ReportWebService\Services\ReportInfoManager;

class ReportGenerator extends Simple{

    protected $reportInfoManager;
    protected $dataSet;
    protected $metaData;
    protected $filter;

    public function __construct($reportPath, ReportInfoManager $reportInfoManager){
        $this->reportPath = $reportPath;
        $this->reportInfoManager = $reportInfoManager;
    }

    /**
     * Get the message to be sent on the response.
     *
     * Save the row on the session and put it on the get request that will be made.
     *
     * @param string $fileName File generated.
     * @returns string Message to be sent on the response.
     */
    public function getMessage($fileName){
        $row = array("fileName"=>$fileName, "title"=>$this->metaData['title']);
        $key = $this->reportInfoManager->save($row);
        return "Report?requestType=Row&row[KEY]=".$key;
    }

    /**
     * Get the necessary params to generate the report.
     *
     * Set the variables used by all functions that generate reports - $this->dataSet and $this->metaData.
     *
     * @param Row $request Request with params to generate the report.
     */
    protected function getParams(Row $request){
        $row = $request->getRow();
        $this->dataSet = json_decode($row["dataSet"], true);
        $this->metaData = json_decode($row["metaData"], true);
        $this->filter = json_decode($row["filter"], true);
    }

    /**
     * Generates the pdf.
     *
     * Gets filename and path, validate the metadata, get a new SplFileObject and generates the 
     * report.
     *
     * @param Row $request          The request's row.
     * @param Response  $response   The response that will be sent.
     */
    public function generatePDF(Row $request, Response $response) {
        $this->getParams($request);
        $reportBuilder = new Report($this->reportPath);
        try {
            $fileName = $reportBuilder->generateReport($this->metaData, $this->dataSet, $this->filter, true);
            $message = $this->getMessage($fileName);
            $response->addMessage(new Response\Message($message));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Creates the XLS file.
     *
     * Get the params and generate the XLS file, testing if there were any errors on the report generation.
     *
     * @param Row $request          The request's row.
     * @param Response  $response   The response that will be sent.
     */
    public function generateXLS(Row $request, Response $response) {
        $this->getParams($request);
        $reportBuilder = new XLSXWriter($this->reportPath);

        try {
            $fileName = $reportBuilder->generateXLS($this->dataSet,$this->metaData);
            $message = $this->getMessage($fileName);
            $response->addMessage(new Response\Message($message));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Creates the CSV file.
     *
     * Get the params and generate the CSV file, testing if there were any errors on the report generation.
     *
     * @param Row $request          The request's row.
     * @param Response  $response   The response that will be sent.
     */
    public function generateCSV(Row $request, Response $response) {
        $this->getParams($request);
        $reportBuilder = new CSVService($this->reportPath);

        try {
            $fileName = $reportBuilder->generateCSV($this->dataSet,$this->metaData);
            $message = $this->getMessage($fileName);
            $response->addMessage(new Response\Message($message));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}