<?php
namespace Zeedhi\ReportWebService\SpreadSheet;

/**
 * The xls writer.
 */
class XLSXWriter implements \Zeedhi\ReportWebService\SpreadSheet\SpreadsheetWriter {

    /** @var $reportDirectory The directory of the report. */
    protected $reportDirectory;
    /** @var array $numberFormats The existing formats for numbers. */
    protected $numberFormats = array(
        "currency" => true,
        "float" => true,
        "number" => true,
        "metric" => true,
        "percent" => true
    );

    /**
     * Constructor.
     *
     * @param $reportDirectory The directory of the report.
     */
    function __construct($reportDirectory){
        $this->reportDirectory = $reportDirectory;
    }

    /**
     * Generates the XLS.
     *
     * @param $gridData The grid data.
     * @param $metaData The meta data to build the report.
     *
     * @return The generated file name.
     */
    public function generateXLS($gridData, $metaData){
        $fileName = uniqid() . ".xlsx";
        $fileName = str_replace(' ', '', $fileName);
        
        $columns = $metaData['columns'];
        $writer = new \XLSXWriter();
        
        $header = $this->getHeader($metaData, $columns);
        $this->formatByPrecisions($gridData, $metaData, $columns);
        
        if(!empty($header)){
            unset($gridData[0]);
            $writer->writeSheet($gridData, '', $header);
        }
        else{
            $gridData[0] = array_map(
                function($column) use ($columns){
                    return isset($columns[$column]['description']) ? $columns[$column]['description'] : "";
                }, 
                $gridData[0]
            );
            $writer->writeSheet($gridData);
        }
        $writer->writeToFile($this->reportDirectory."/".$fileName);
        
        return $fileName;
    }
    
    /**
     * Formats the grid data by precision.
     *
     * @param $gridData The grid data.
     * @param $metaData The meta data to build the report.
     * @param $columns The grid columns.
     */
    private function formatByPrecisions(&$gridData, $metaData, $columns){
        $precisions = $this->extractPrecisions($columns, $gridData[0]);
        for($i=1; $i < count($gridData); $i++){
            foreach($precisions as $key => $precision){
                $gridData[$i][$key] = round($gridData[$i][$key], $precision);
            }
        }
    }

    /**
     * Extracts the precisions from the columns.
     *
     * @param $columns The grid columns.
     * @param fieldNames The name of the fields.
     *
     * @return The generated file name.
     */
    private function extractPrecisions($columns, $fieldNames){
        $precisions = array();
        foreach($columns as $columnName => $column){
            if($this->columnHasNumberFormat($column)){
                $columnFormat = $column['format'];
                $precision = 2;
                if(isset($columnFormat['params']) && !empty($columnFormat['params']['precision'])){
                    $precision = $columnFormat['params']['precision'];
                }
                $precisions[array_search($columnName, $fieldNames)] = $precision;
            }
        }
        return $precisions;
    }

    /**
     * Verifies if the column has a number format.
     *
     * @param $column the columns.
     *
     * @return True if the columns has a number format.
     */
    private function columnHasNumberFormat($column){
        $hasNumberFormat = false;
        if (isset($column['format'])){
            $formatType = $column['format']['type'];
            $hasNumberFormat = isset($this->numberFormats[$formatType]) && $this->numberFormats[$formatType];
        }
        return $hasNumberFormat;
    }

    /**
     * Retrieves the header from the metada data.
     *
     * @param $metaData The meta data to build the report.
     * @param $columns The grid columns.
     *
     * @return The generated file name.
     */
    private function getHeader(array $metaData, array $columns){
        $preparedHeader = [];
        if(isset($metaData['xlsDefaultType'])){
            foreach($columns as $column){
                $preparedHeader[$column['description']] = key_exists('xlsType', $column) ? 
                $column['xlsType'] : $metaData['xlsDefaultType'];
            }
            $columns = $preparedHeader;
        }
        return $preparedHeader;
    }
}